/*
 * Example taken from the slides:
 * www.dmi.unipg.it/~bista/didattica/CSP/soft-constraints.ppt pag 12
 * prof. Bistarelli - Sistemi con Vincoli
 */
import JaCoP.core.IntVar;
import JaCoP.search.*;
import SoftLib.constraints.*;
import SoftLib.core.SoftStore;
import SoftLib.core.SoftStore.Semiring;
import SoftLib.weight.WeightExtensional;
import SoftLib.weight.WeightUnaryExtensional;

public class Esempio_SoftLib {

    public static void main(String[] args) {
        /*
         * specify the semiring type in the SoftStore construnctor:
         *  - Semiring.FUZZY
         *  - Semiring.PROBABILISTIC
         *  - Semiring.WEIGHTED
         * 
         * it's possible to modify this parameter to see the related solution.
         */
        SoftStore store = new SoftStore(Semiring.FUZZY);
        
        /*
         * In this example two variables are created (X and Y) with a domain (0..2) where:
         *  - a = 0
         *  - b = 1
         *  - c = 2
         * 
         */
        IntVar[] list  = new IntVar[2],
                 listX = new IntVar[1],
                 listY = new IntVar[1];
        list[0] = listX[0] = new IntVar(store, "X", 0, 2);
        list[1] = listY[0] = new IntVar(store, "Y", 0, 2);

        /*
         * A constraint with all the possible solutions for the couple (X,Y) is imposed together with the related costs expressed in millesimals
         * The solutions are all the possible assignment for the two variables, the costs are contained in the following array.
         * 
         *  - {0,0} costs 0.8
         *  - {0,1} costs 0.2
         *  - {0,2} costs 0,2
         *  etc...
         */
        int[] w = new int[]{800, 200, 200, 0, 0, 100, 800, 200, 200};
        store.impose(new SoftExtensionalSupportSTR(list , new int[][]{{0,0}, {0,1}, {0,2}, {1,0}, {1,1}, {1,2}, {2,0}, {2,1}, {2,2}}), new WeightExtensional(w));

        /*
         * Here the constraint with the admissible solution for the X variable and the costs in millesimals are imposed.
         * The costs for each assignment are:
         *  - {0} costs 0.9
         *  - {1} costs 0.1
         *  - {2} costs 0.9
         */
        int[] w2 = new int[]{900, 100, 900};
        store.impose(new SoftXltC(list[0], 2), new WeightUnaryExtensional(w2));
        
        /*
         * As above for the Y variable
         */
        int[] w3 = new int[]{900, 500, 500};
        store.impose(new SoftXltC(list[1], 2), new WeightUnaryExtensional(w3));
        
        /*
         * The costs constraints need to be imposed using imposeWeights() methos.
         */
        store.imposeWeights();
        
        /*
         * Call to the search function.
         */
        Search<IntVar> label = new DepthFirstSearch<IntVar>();
        SelectChoicePoint<IntVar> select = new SimpleSelect<IntVar>(list,
                new SmallestDomain<IntVar>(),
                new IndomainMin<IntVar>());

        label.getSolutionListener().searchAll(true);
        label.getSolutionListener().recordSolutions(true);

        /*
         * The labelling is called and stored.getCost() is passed that is the variable to minimize/maximize.
         * In the FUZZY case, the solution (X=0, Y=0), or (X=a, Y=a) in the slides, is returned with a cost of -800, 0.8 in the slides,
         * and it is specificied that there are two optimal solutions.
         * In the PROBABILISTIC case, the solution (X=0, Y=0), or (X=a, Y=a) in the slides, is returned with a cost of -648, 0.648 in the slides,
         * and it is specificied that there are two optimal solutions.
         */
        label.labeling(store, select, store.getCost());

        /*
         * To search for all the optimal solutions it is necessary to comment the line above and uncomment the two following lines.
         * The optimal cost, obtained using the search in the previous way, is then passed as second parameter in the XeqC constraint.
         */
        //store.impose(new XeqC(store.getCost(),-648));
        //label.labeling(store, select);
        
        /*
         * The following loop prints all the solutions
         */
        for (int i = 1; i <= label.getSolutionListener().solutionsNo(); i++) {
            System.out.print("Soluzione " + i + ": ");
            for (int j = 0; j < label.getSolution(i).length; j++) {
                System.out.print(label.getSolution(i)[j]);
            }
            System.out.println();
        }        
    }
    
}