package SoftLib.weight;

import JaCoP.constraints.Constraint;
import JaCoP.constraints.Distance;
import JaCoP.core.IntVar;
import SoftLib.core.SoftStore;
import SoftLib.core.SoftStore.Semiring;

/**
 * This class imposes the cost on a constraint using the Distance constraint.
 * There are two setWeight methods to impose the cost constraint:
 * - the first one has to be used with primitive constraints and binary constraints
 *   and takes the two variables as parameters
 * - the second one has to be used with global constraint and takes the 
 *   constraint as parameter in order to allow access to the costs list
 */

public class WeightDistance extends PrimitiveBinaryWeight {

    public WeightDistance() {
    }
    
    /*
     * method usable with primitive constraint and binary constraint:
     * it sets the distance between the two variables as the cost of the constraint
     */
    @Override
    public Constraint setWeight(SoftStore store, IntVar x, IntVar y) {

        int[] gap = calcGap(x, y);
        if((store.getSrType() == Semiring.FUZZY ||
            store.getSrType() == Semiring.PROBABILISTIC) &&
            gap[1] > 1000) {
            throw new UnsupportedOperationException("In " + ((store.getSrType() == Semiring.FUZZY) ? "Fuzzy" : "Probabilistic") + " semiring the probability/preference value of an assignment cannot be greater than 1000");
        }
        IntVar z = new IntVar(store, "cost", gap[0], gap[1]);
        store.costs.add(z);

        Constraint cost = new Distance(x, y, z);
        return cost;
    }
    
    /*
     * the calcGap method is usefull to avoid overflow.
     * calcGap computes the interval [min,max] of the variable in which the cost will be stored:
     * there is no need for all the available space of the Int type (using the Int type there
     * is the posssibility to generate uncontrolled overflow)
     */
    private int[] calcGap(IntVar x, IntVar y) {
        int xMin = x.min();
        int xMax = x.max();
        int yMin = y.min();
        int yMax = y.max();
        int minGap, maxGap;

        if (yMin < xMin) {     //                                y     x
            if (yMax < xMin) { //(yMin < xMin && yMax < xMin): <...> [...] 
                minGap = Math.abs(yMax - xMin);
                maxGap = Math.abs(yMin - xMax);
            }
            else if (yMax <= xMax) { //(yMin < xMin && yMax >= xMin && yMax <= xMax): <..[..>..]
                minGap = 0;
                maxGap = Math.abs(yMin - xMax);
            }
            else { //(yMin < xMin && yMax >= xMin && yMax > xMax) : <..[.]..>
                minGap = 0;
                maxGap = Math.max(Math.abs(yMin - xMax), Math.abs(yMax - xMin));
            }
        }
        else {                 //                                 x     y
            if (xMax < yMin) { //(yMin >= xMin && xMax < yMin): [...] <...>
                minGap = Math.abs(xMax - yMin);
                maxGap = Math.abs(xMin - yMax);
            }
            else if (xMax <= yMax) { //(yMin >= xMin && xMax >= yMin && xMax <= yMax): [..<..]..>
                minGap = 0;
                maxGap = Math.abs(xMin - yMax);
            }
            else { // (yMin >= xMin && xMax >= yMin && xMax > xMax): [..<.>..]
                minGap = 0;
                maxGap = Math.max(Math.abs(xMin - yMax), Math.abs(xMax - yMin));
            }
        }

        int ret[] = {minGap, maxGap};
        return ret;
    }

    @Override
    public int evaluateWeight(int x, int y) {
        return Math.abs(x-y);
    }
}