package SoftLib.weight;

import JaCoP.constraints.Constraint;
import JaCoP.constraints.ExtensionalSupportSTR;
import JaCoP.core.IntVar;
import JaCoP.core.Interval;
import SoftLib.core.IntCouple;
import SoftLib.core.SoftStore;
import java.util.HashMap;

/**
 * This class permits to specify, for each couple of value assigned to the variables involved, the related weight.
 * To use this class it is necessary to specify the weight/preference/probability value for each possible combination
 * of a variable from the domain of the first variable with a value from the second variable's domain.
 * An ExtensionalSupport constraint is used to impose the weight function.
 */

public class WeightExtensional extends PrimitiveBinaryWeight {
    
    private int[] w;
    private HashMap<IntCouple, Integer> values;

    public WeightExtensional (int[] w) {
        this.w = w;
        this.values = new HashMap<IntCouple, Integer>();
    }
    
    @Override
    public Constraint setWeight(SoftStore store, IntVar x, IntVar y) {
        int zDomSize = x.getSize()*y.getSize();
        if (w.length != zDomSize) {
            throw new IllegalArgumentException("Wrong weights dimension");
        }
        int max = w[0], min = w[0];
        for(int i=1; i < w.length; i++) {
            if(w[i] > max) {max = w[i];}
            if(w[i] < min) {min = w[i];}
        }
        if((store.getSrType() == SoftStore.Semiring.FUZZY ||
            store.getSrType() == SoftStore.Semiring.PROBABILISTIC) &&
            (max > 1000 || min < 0)) {
            throw new UnsupportedOperationException("In " + ((store.getSrType() == SoftStore.Semiring.FUZZY) ? "Fuzzy" : "Probabilistic") + " semiring the probability/preference value of an assignment cannot be less than 0 or greater than 1000");
        }
        int[][] triples = new int[(x.getSize()*y.getSize())][3];        
        Interval intX, intY;
        int index = 0;
        for (int i=0; ((intX = x.dom().getInterval(i)) != null); i++) {
            for (int xValue = intX.min(); xValue <= intX.max(); xValue++) {
                for (int j = 0; ((intY = y.dom().getInterval(j)) != null); j++) {
                    for (int yValue = intY.min(); yValue <= intY.max(); yValue++) {
                        triples[index][0] = xValue;
                        triples[index][1] = yValue;
                        triples[index][2] = w[index];
                        values.put(new IntCouple(xValue, yValue), w[index]);
                        index++;
                    }
                }
            }
        }
        IntVar z = new IntVar(store, min, max);
        store.costs.add(z);
        IntVar[] v = {x, y, z};
        Constraint cost = new ExtensionalSupportSTR(v, triples);
        return cost;
    }
    
    @Override
    public int evaluateWeight(int x, int y) {
        IntCouple key = new IntCouple(x, y);
        if(values.containsKey(key)) {
            return values.get(key);    
        }else{
            throw new IllegalArgumentException("x value is not in the X domain or y is not in the Y domain");
        }
    }
    
}